@@ -5,15 +5,14 @@ test_that("can link function calls", {
   )
 
   expect_equal(href_expr_(foo()), "bar.html")
-  expect_equal(href_expr_(foo(1, 2, 3)), "bar.html")
   # even if namespaced
   expect_equal(href_expr_(test::foo()), "bar.html")
-  expect_equal(href_expr_(test::foo(1, 2, 3)), "bar.html")
 
-  # but function factories are ignored
+  # but functions with arguments are ignored
+  expect_equal(href_expr_(foo(1, 2, 3)), NA_character_)
+  # as are function factories are ignored
   expect_equal(href_expr_(foo()(1, 2, 3)), NA_character_)
-  # as functions with special syntax
-  expect_equal(href_expr_(if (TRUE) 1), NA_character_)
+  expect_equal(href_expr_(test::foo()(1, 2, 3)), NA_character_)
 })
 
 test_that("base function calls linked", {
@@ -39,16 +38,32 @@ test_that("can link remote objects", {
   expect_equal(href_expr_(MASS::abbey), href_topic_remote("abbey", "MASS"))
   expect_equal(href_expr_(MASS::addterm()), href_topic_remote("addterm", "MASS"))
   expect_equal(href_expr_(MASS::addterm.default()), href_topic_remote("addterm", "MASS"))
+  expect_equal(href_expr_(base::`::`), href_topic_remote("::", "base"))
 
   # Doesn't exist
   expect_equal(href_expr_(MASS::blah), NA_character_)
 })
 
-test_that("can link to functions in registered packages", {
+test_that("can link to topics in registered packages", {
   local_options("downlit.attached" = "MASS")
 
   expect_equal(href_expr_(addterm()), href_topic_remote("addterm", "MASS"))
-  expect_equal(href_expr_(addterm.default()), href_topic_remote("addterm", "MASS"))
+  expect_equal(href_expr_(?abbey), href_topic_remote("abbey", "MASS"))
+
+  # but has to be a function
+  expect_equal(href_expr_(abbey()), NA_character_)
+})
+
+test_that("can link to package names in registered packages", {
+  expect_equal(
+    autolink_curly("{downlit}"),
+    "<a href='https://downlit.r-lib.org/'>downlit</a>"
+  )
+
+  expect_equal(autolink_curly("{package}"), NA_character_)
+
+  # No curly = no link
+  expect_equal(autolink_curly(""), NA_character_)
 })
 
 test_that("can link to functions in base packages", {
@@ -79,8 +94,8 @@ test_that("or local sites, if registered", {
   expect_equal(href_expr_(MASS::abbey), "MASS/reference/abbey.html")
 })
 
-test_that("only links bare symbols if they're infix functions", {
-  expect_equal(autolink_url("%in%"), "https://rdrr.io/r/base/match.html")
+test_that("bare bare symbols are not linked", {
+  expect_equal(autolink_url("%in%"), NA_character_)
   expect_equal(autolink_url("foo"), NA_character_)
 })
 
@@ -88,6 +103,7 @@ test_that("returns NA for bad inputs", {
   expect_equal(autolink_url(""), NA_character_)
   expect_equal(autolink_url("a; b"), NA_character_)
   expect_equal(autolink_url("1"), NA_character_)
+  expect_equal(autolink_url("ls *t??ne.pb"), NA_character_)
 })
 
 # help --------------------------------------------------------------------
@@ -113,7 +129,8 @@ test_that("can link help calls", {
   expect_equal(href_expr_(help("foo")), "foo.html")
   expect_equal(href_expr_(help("foo", "test")), "foo.html")
   expect_equal(href_expr_(help(package = "MASS")), "https://rdrr.io/pkg/MASS/man")
-  expect_equal(href_expr_(help()), NA_character_)
+  expect_equal(href_expr_(help()), "https://rdrr.io/r/utils/help.html")
+  expect_equal(href_expr_(help(a$b)), NA_character_)
 })
 
 # library and friends -----------------------------------------------------
@@ -122,11 +139,17 @@ test_that("library() linked to package reference", {
   skip_on_cran() # in case URLs change
   skip_on_os("solaris")
 
-  expect_equal(href_expr_(library()), "https://rdrr.io/r/base/library.html")
-  expect_equal(href_expr_(library(rlang)), "http://rlang.r-lib.org")
+  expect_equal(href_expr_(library(rlang)), "https://rlang.r-lib.org")
   expect_equal(href_expr_(library(MASS)), "http://www.stats.ox.ac.uk/pub/MASS4/")
 })
 
+test_that("except when not possible", {
+  expect_equal(href_expr_(library()), "https://rdrr.io/r/base/library.html")
+  expect_equal(href_expr_(library(doesntexist)), "https://rdrr.io/r/base/library.html")
+  expect_equal(href_expr_(library(package = )), "https://rdrr.io/r/base/library.html")
+  expect_equal(href_expr_(library("x", "y", "z")), "https://rdrr.io/r/base/library.html")
+})
+
 # vignette ----------------------------------------------------------------
 
 test_that("can link to local articles", {
@@ -141,6 +164,15 @@ test_that("can link to local articles", {
   expect_equal(href_expr_(vignette("y")), NA_character_)
 })
 
+test_that("can link to bioconductor vignettes", {
+  skip_if_not_installed("MassSpecWavelet")
+
+  expect_equal(
+    href_expr_(vignette("MassSpecWavelet", "MassSpecWavelet")),
+    "https://bioconductor.org/packages/release/bioc/vignettes/MassSpecWavelet/inst/doc/MassSpecWavelet.html"
+  )
+})
+
 test_that("can link to remote articles", {
   skip_on_cran()
 
@@ -166,6 +198,19 @@ test_that("or local sites, if registered", {
   expect_equal(href_expr_(vignette("sha1", "digest")), "digest/articles/sha1.html")
 })
 
+test_that("looks in attached packages", {
+  local_options("downlit.attached" = c("grid", "digest"))
+
+  expect_equal(
+    href_expr_(vignette("sha1")),
+    "https://cran.rstudio.com/web/packages/digest/vignettes/sha1.html"
+  )
+  expect_equal(
+    href_expr_(vignette("moveline")),
+    "https://cran.rstudio.com/web/packages/grid/vignettes/moveline.pdf"
+  )
+})
+
 test_that("fail gracefully with non-working calls", {
   expect_equal(href_expr_(vignette()), "https://rdrr.io/r/utils/vignette.html")
   expect_equal(href_expr_(vignette(package = package)), NA_character_)
